@@ -22,6 +22,8 @@
 int numEvals ;
 int numCacheHits ;
 
+int  bStrict ; /* aply use strict in each eval */
+
 /* -------------------------------------------------------------------------------
 *
 * Eval PERL Statements 
@@ -31,8 +33,118 @@ int numCacheHits ;
 *
 ------------------------------------------------------------------------------- */
 
+int EvalDirect (/*in*/    SV * pArg) 
+    {
+    dSP;
+    
+    EPENTRY (EvalDirect) ;
+
+    tainted = 0 ;
+
+    PUSHMARK(sp);
+    perl_eval_sv(pArg, G_SCALAR | G_KEEPERR);
+
+
+    if (SvTRUE (GvSV(errgv)))
+         {
+         STRLEN l ;
+         char * p = SvPV (GvSV(errgv), l) ;
+         if (l > sizeof (errdat1) - 1)
+             l = sizeof (errdat1) - 1 ;
+         strncpy (errdat1, p, l) ;
+         if (l > 0 && errdat1[l-1] == '\n')
+             l-- ;
+         errdat1[l] = '\0' ;
+         
+	 LogError (rcEvalErr) ;
+
+	 sv_setpv(GvSV(errgv),"");
+         return rcEvalErr ;
+         }
+
+    return ok ;
+    }
+
+
 #define EVAL_SUB
 
+#ifndef EVAL_BY_SUB
+
+/* -------------------------------------------------------------------------------
+*
+* Eval PERL Statements into a sub 
+* 
+* in  sArg   Statement to eval
+* out pRet   pointer to SV contains an CV to the evaled code
+*
+------------------------------------------------------------------------------- */
+
+static int EvalAll (/*in*/  const char *  sArg,
+                    /*out*/ SV **         pRet)             
+    {
+    static char sFormat []       = "package %s ; sub { %s }" ;
+    static char sFormatStrict [] = "package %s ; use strict ; sub { %s }" ; 
+    SV *   pSVCmd ;
+    dSP;
+    
+    EPENTRY (EvalAll) ;
+
+    if (bDebug & dbgDefEval)
+        lprintf ("[%d]DEF:  %s\n", nPid, sArg) ;
+
+    tainted = 0 ;
+
+    if (bStrict)
+        pSVCmd = newSVpvf(sFormatStrict, sEvalPackage,  sArg) ;
+    else
+        pSVCmd = newSVpvf(sFormat, sEvalPackage,  sArg) ;
+
+    PUSHMARK(sp);
+    perl_eval_sv(pSVCmd, G_SCALAR | G_KEEPERR);
+    SvREFCNT_dec(pSVCmd);
+
+    SPAGAIN;
+    *pRet = POPs;
+    PUTBACK;
+
+    if (bDebug & dbgMem)
+        lprintf ("[%d]SVs:  %d\n", nPid, sv_count) ;
+    
+    if (SvTRUE (GvSV(errgv)))
+         {
+         STRLEN l ;
+         char * p = SvPV (GvSV(errgv), l) ;
+         if (l > sizeof (errdat1) - 1)
+             l = sizeof (errdat1) - 1 ;
+         strncpy (errdat1, p, l) ;
+         if (l > 0 && errdat1[l-1] == '\n')
+             l-- ;
+         errdat1[l] = '\0' ;
+         
+	 *pRet = newSVpv (LogError (rcEvalErr), 0) ;
+
+	 sv_setpv(GvSV(errgv),"");
+         return rcEvalErr ;
+         }
+
+    return ok ;
+    }
+
+#else
+
+/* -------------------------------------------------------------------------------
+*
+* Eval PERL Statements 
+* 
+* in  sArg   Statement to eval
+* out pRet   pointer to SV contains an CV to the evaled code
+*
+* This function do it by calling a perl subroutine, this was the onyl way in 
+* perl 5.003, because the function perl_eval_sv was broken
+*
+------------------------------------------------------------------------------- */
+
+
 static int EvalAll (/*in*/  const char *  sArg,
                     /*out*/ SV **         pRet)             
     {
@@ -42,7 +154,7 @@ static int EvalAll (/*in*/  const char *  sArg,
 #endif
     dSP;                            /* initialize stack pointer      */
 
-    EPENTRY (EvalAll) ;
+    EPENTRY (EvalAll-BySub) ;
 
     if (bDebug & dbgDefEval)
         lprintf ("[%d]DEF:  %s\n", nPid, sArg) ;
@@ -56,7 +168,7 @@ static int EvalAll (/*in*/  const char *  sArg,
     PUSHMARK(sp);                   /* remember the stack pointer    */
     XPUSHs(sv_2mortal(newSVpv((char *)sArg, strlen (sArg)))); /* push the base onto the stack  */
     PUTBACK;                        /* make local stack pointer global */
-    num = perl_call_pv ("HTML::Embperl::_evalsub_", G_SCALAR | G_EVAL) ; /* call the function             */
+    num = perl_call_pv ("_evalsub_", G_SCALAR | G_EVAL) ; /* call the function             */
 #else
     
     pSVArg = sv_2mortal(newSVpv((char *)sArg, strlen (sArg))) ;
@@ -93,7 +205,8 @@ static int EvalAll (/*in*/  const char *  sArg,
          {
          STRLEN l ;
          char * p = SvPV (GvSV(errgv), l) ;
-         l = min (l, sizeof (errdat1) - 1) ;
+         if (l > sizeof (errdat1) - 1))
+             l = sizeof (errdat1) - 1) ;
          strncpy (errdat1, p, l) ;
          errdat1[l] = '\0' ;
          LogError (rcEvalErr) ;
@@ -103,88 +216,7 @@ static int EvalAll (/*in*/  const char *  sArg,
     return num ;
     }
 
-
-/* -------------------------------------------------------------------------------
-*
-* Eval PERL Statements in safe namespace 
-* 
-* in  sArg   Statement to eval
-* out pRet   pointer to SV contains an CV to the evaled code
-*
-------------------------------------------------------------------------------- */
-
-
-static int EvalSafe (/*in*/  const char *  sArg,
-                     /*out*/ SV **         pRet)             
-    {
-    int   num ;         
-    int   nCountUsed = TableState.nCountUsed ;
-    int   nRowUsed   = TableState.nRowUsed ;
-    int   nColUsed   = TableState.nColUsed ;
-    dSP;                            /* initialize stack pointer      */
-
-    EPENTRY (EvalSafe) ;
-
-    if (bDebug & dbgEval)
-        lprintf ("[%d]EVAL< %s\n", nPid, sArg) ;
-
-
-    ENTER;                          /* everything created after here */
-    SAVETMPS;                       /* ...is a temporary variable.   */
-    PUSHMARK(sp);                   /* remember the stack pointer    */
-    XPUSHs(pNameSpace) ;
-    XPUSHs(sv_2mortal(newSVpv((char *)sArg, strlen (sArg)))); 
-    PUTBACK;                        /* make local stack pointer global */
-    num = perl_call_method ("reval", G_SCALAR | G_EVAL) ;  /* call the function             */
-    SPAGAIN;                        /* refresh stack pointer         */
-    
-    if (bDebug & dbgMem)
-        lprintf ("[%d]SVs:  %d\n", nPid, sv_count) ;
-    /* pop the return value from stack */
-    if (num == 1)   
-        {
-        *pRet = POPs ;
-        SvREFCNT_inc (*pRet) ;
-        if ((nCountUsed != TableState.nCountUsed ||
-             nColUsed != TableState.nColUsed ||
-             nRowUsed != TableState.nRowUsed) &&
-              !SvOK (*pRet))
-            TableState.nResult = 0 ;
-
-        if ((bDebug & dbgTab) &&
-            (TableState.nCountUsed ||
-             TableState.nColUsed ||
-             TableState.nRowUsed))
-            lprintf ("[%d]TAB:  nResult = %d\n", nPid, TableState.nResult) ;
-
-        if (bDebug & dbgEval)
-            if (SvOK (*pRet))
-                lprintf ("[%d]EVAL> %s\n", nPid, SvPV (*pRet, na)) ;
-            else
-                lprintf ("[%d]EVAL> <undefined>\n",nPid) ;
-        }
-     else
-        {
-        *pRet = NULL ;
-        if (bDebug & dbgEval)
-            lprintf ("[%d]EVAL> <NULL>\n", nPid) ;
-        }
-
-    PUTBACK;
-
-     if (SvTRUE (GvSV(errgv)))
-         {
-         strncpy (errdat1, SvPV (GvSV(errgv), na), sizeof (errdat1) - 1) ;
-         LogError (rcEvalErr) ;
-         }
-
-    
-    
-    FREETMPS;                       /* free that return value        */
-    LEAVE;                       /* ...and the XPUSHed "mortal" args.*/
-    
-    return num ;
-    }
+#endif /* EVAL_BY_SUB */
 
 /* -------------------------------------------------------------------------------
 *
@@ -221,7 +253,7 @@ static int EvalAllNoCache (/*in*/  const char *  sArg,
     PUSHMARK(sp);                   /* remember the stack pointer    */
     XPUSHs(sv_2mortal(newSVpv((char *)sArg, strlen (sArg)))); /* push the base onto the stack  */
     PUTBACK;                        /* make local stack pointer global */
-    num = perl_call_pv ("HTML::Embperl::_eval_", G_SCALAR /*| G_EVAL*/) ; /* call the function             */
+    num = perl_call_pv ("_eval_", G_SCALAR /*| G_EVAL*/) ; /* call the function             */
 #else
     
     pSVArg = sv_2mortal(newSVpv((char *)sArg, strlen (sArg))) ;
@@ -392,8 +424,19 @@ static int CallCV  (/*in*/  const char *  sArg,
 
      if (SvTRUE (GvSV(errgv)))
          {
-         strncpy (errdat1, SvPV (GvSV(errgv), na), sizeof (errdat1) - 1) ;
-         LogError (rcEvalErr) ;
+         STRLEN l ;
+         char * p = SvPV (GvSV(errgv), l) ;
+         if (l > sizeof (errdat1) - 1)
+             l = sizeof (errdat1) - 1 ;
+         strncpy (errdat1, p, l) ;
+         if (l > 0 && errdat1[l-1] == '\n')
+             l-- ;
+         errdat1[l] = '\0' ;
+         
+	 LogError (rcEvalErr) ;
+
+	 sv_setpv(GvSV(errgv),"");
+         return rcEvalErr ;
          }
 
      if (bDebug & dbgWatchScalar)
@@ -420,16 +463,24 @@ static int EvalAndCall (/*in*/  const char *  sArg,
 
     {
     int     rc ;
+    SV *   pSub ;
     
     
     EPENTRY (EvalAndCall) ;
 
-    rc = EvalAll (sArg, pRet) ;
+    rc = EvalAll (sArg, &pSub) ;
 
-    if (rc == ok && *pRet != NULL && SvTYPE (*pRet) == SVt_RV)
-        *ppSV = SvRV(*pRet) ;
+    if (rc == ok && pSub != NULL && SvTYPE (pSub) == SVt_RV)
+        {
+        *ppSV = SvRV(pSub) ;
+        SvREFCNT_inc (*ppSV) ;
+        }
     else
+        {
+        if (pSub != NULL && SvTYPE (pSub) == SVt_PV)
+            *ppSV = pSub ; /* save error message */
         return rc ;
+        }
 
     if (*ppSV && SvTYPE (*ppSV) == SVt_PVCV)
         { /* Call the compiled eval */
@@ -470,17 +521,20 @@ int Eval (/*in*/  const char *  sArg,
     if (bDebug & dbgCacheDisable)
         return EvalAllNoCache (sArg, pRet) ;
 
-    if (bSafeEval)
-        return EvalSafe (sArg, pRet) ;
-
     /* Already compiled ? */
 
     ppSV = hv_fetch(pCacheHash, (char *)&nFilepos, sizeof (nFilepos), 1) ;  
     if (ppSV == NULL)
         return rcHashError ;
 
+    if (*ppSV != NULL && SvTYPE (*ppSV) == SVt_PV)
+        {
+        strncpy (errdat1, SvPV(*ppSV, na), sizeof (errdat1) - 1) ; 
+        LogError (rcEvalErr) ;
+        return rcEvalErr ;
+        }
 
-    if (SvTYPE (*ppSV) != SVt_PVCV)
+    if (*ppSV == NULL || SvTYPE (*ppSV) != SVt_PVCV)
         return EvalAndCall (sArg, ppSV, pRet) ;
 
     numCacheHits++ ;
@@ -518,16 +572,20 @@ int EvalTrans (/*in*/  char *   sArg,
     if (bDebug & dbgCacheDisable)
         return EvalAllNoCache (sArg, pRet) ;
 
-    if (bSafeEval)
-        return EvalSafe (sArg, pRet) ;
-
     /* Already compiled ? */
 
     ppSV = hv_fetch(pCacheHash, (char *)&nFilepos, sizeof (nFilepos), 1) ;  
     if (ppSV == NULL)
         return rcHashError ;
 
-    if (SvTYPE (*ppSV) != SVt_PVCV)
+    if (*ppSV != NULL && SvTYPE (*ppSV) == SVt_PV)
+        {
+        strncpy (errdat1, SvPV(*ppSV, na), sizeof (errdat1) - 1) ; 
+        LogError (rcEvalErr) ;
+        return rcEvalErr ;
+        }
+
+    if (*ppSV == NULL || SvTYPE (*ppSV) != SVt_PVCV)
         {
         /* strip off all <HTML> Tags */
         TransHtml (sArg) ;
@@ -564,7 +622,7 @@ int EvalNum (/*in*/  char *        sArg,
     EPENTRY (EvalNum) ;
 
 
-    n = EvalTrans (sArg, nFilepos, &pRet) ;
+    n = Eval (sArg, nFilepos, &pRet) ;
     
     if (pRet)
         {
@@ -572,7 +630,7 @@ int EvalNum (/*in*/  char *        sArg,
         SvREFCNT_dec (pRet) ;
         }
     else
-        pNum = 0 ;
+        *pNum = 0 ;
 
     return ok ;
     }
